@@ -17,15 +17,19 @@ func Test(t *testing.T) {
 		"bloop",
 		"efaceany",
 		"fmtappendf",
+		"forvar",
 		"mapsloop",
 		"minmax",
 		"omitzero",
 		"rangeint",
 		"slicescontains",
 		"slicesdelete",
+		"stringscutprefix",
+		"stringscutprefix/bytescutprefix",
 		"splitseq",
 		"fieldsseq",
 		"sortslice",
 		"testingcontext",
+		"waitgroup",
 	)
 }
