@@ -95,7 +95,7 @@ func ExtractToNewFile(ctx context.Context, snapshot *cache.Snapshot, fh file.Han
 
 	start, end, firstSymbol, ok := selectedToplevelDecls(pgf, start, end)
 	if !ok {
-		return nil, bug.Errorf("invalid selection")
+		return nil, fmt.Errorf("invalid selection")
 	}
 	pgf.CheckPos(start) // #70553
 	// Inv: start is valid wrt pgf.Tok.
@@ -138,7 +138,7 @@ func ExtractToNewFile(ctx context.Context, snapshot *cache.Snapshot, fh file.Han
 	}
 
 	var buf bytes.Buffer
-	if c := copyrightComment(pgf.File); c != nil {
+	if c := CopyrightComment(pgf.File); c != nil {
 		start, end, err := pgf.NodeOffsets(c)
 		if err != nil {
 			return nil, err
