@@ -17,7 +17,6 @@ package golang
 // - list promoted methods---we have type information!
 // - gather Example tests, following go/doc and pkgsite.
 // - add option for doc.AllDecls: show non-exported symbols too.
-// - abbreviate long signatures by replacing parameters 4 onwards with "...".
 // - style the <li> bullets in the index as invisible.
 // - add push notifications such as didChange -> reload.
 // - there appears to be a maximum file size beyond which the
@@ -25,9 +24,10 @@ package golang
 // - modify JS httpGET function to give a transient visual indication
 //   when clicking a source link that the editor is being navigated
 //   (in case it doesn't raise itself, like VS Code).
-// - move this into a new package, golang/pkgdoc, and then
+// - move this into a new package, golang/web, and then
 //   split out the various helpers without fear of polluting
-//   the golang package namespace.
+//   the golang package namespace?
+// - show "Deprecated" chip when appropriate.
 
 import (
 	"bytes"
@@ -39,29 +39,244 @@ import (
 	"go/token"
 	"go/types"
 	"html"
-	"log"
 	"path/filepath"
+	"slices"
 	"strings"
 
+	"golang.org/x/tools/go/ast/astutil"
 	"golang.org/x/tools/gopls/internal/cache"
+	"golang.org/x/tools/gopls/internal/cache/parsego"
 	"golang.org/x/tools/gopls/internal/protocol"
-	"golang.org/x/tools/gopls/internal/util/astutil"
+	goplsastutil "golang.org/x/tools/gopls/internal/util/astutil"
 	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/util/safetoken"
-	"golang.org/x/tools/gopls/internal/util/slices"
-	"golang.org/x/tools/gopls/internal/util/typesutil"
+	"golang.org/x/tools/internal/stdlib"
 	"golang.org/x/tools/internal/typesinternal"
 )
 
-// RenderPackageDoc formats the package documentation page.
+// DocFragment finds the package and (optionally) symbol identified by
+// the current selection, and returns the package path and the
+// optional symbol URL fragment (e.g. "#Buffer.Len") for a symbol,
+// along with a title for the code action.
+//
+// It is called once to offer the code action, and again when the
+// command is executed. This is slightly inefficient but ensures that
+// the title and package/symbol logic are consistent in all cases.
+//
+// It returns zeroes if there is nothing to see here (e.g. reference to a builtin).
+func DocFragment(pkg *cache.Package, pgf *parsego.File, start, end token.Pos) (pkgpath PackagePath, fragment, title string) {
+	thing := thingAtPoint(pkg, pgf, start, end)
+
+	makeTitle := func(kind string, imp *types.Package, name string) string {
+		title := "Browse documentation for " + kind + " "
+		if imp != nil && imp != pkg.Types() {
+			title += imp.Name() + "."
+		}
+		return title + name
+	}
+
+	wholePackage := func(pkg *types.Package) (PackagePath, string, string) {
+		// External test packages don't have /pkg doc pages,
+		// so instead show the doc for the package under test.
+		// (This named-based heuristic is imperfect.)
+		if forTest := strings.TrimSuffix(pkg.Path(), "_test"); forTest != pkg.Path() {
+			return PackagePath(forTest), "", makeTitle("package", nil, filepath.Base(forTest))
+		}
+
+		return PackagePath(pkg.Path()), "", makeTitle("package", nil, pkg.Name())
+	}
+
+	// Conceptually, we check cases in the order:
+	// 1. symbol
+	// 2. package
+	// 3. enclosing
+	// but the logic of cases 1 and 3 are identical, hence the odd factoring.
+
+	// Imported package?
+	if thing.pkg != nil && thing.symbol == nil {
+		return wholePackage(thing.pkg)
+	}
+
+	// Symbol?
+	var sym types.Object
+	if thing.symbol != nil {
+		sym = thing.symbol // reference to a symbol
+	} else if thing.enclosing != nil {
+		sym = thing.enclosing // selection is within a declaration of a symbol
+	}
+	if sym == nil {
+		return wholePackage(pkg.Types()) // no symbol
+	}
+
+	// Built-in (error.Error, append or unsafe).
+	// TODO(adonovan): handle builtins in /pkg viewer.
+	if sym.Pkg() == nil {
+		return "", "", "" // nothing to see here
+	}
+	pkgpath = PackagePath(sym.Pkg().Path())
+
+	// Unexported? Show enclosing type or package.
+	if !sym.Exported() {
+		// Unexported method of exported type?
+		if fn, ok := sym.(*types.Func); ok {
+			if recv := fn.Signature().Recv(); recv != nil {
+				_, named := typesinternal.ReceiverNamed(recv)
+				if named != nil && named.Obj().Exported() {
+					sym = named.Obj()
+					goto below
+				}
+			}
+		}
+
+		return wholePackage(sym.Pkg())
+	below:
+	}
+
+	// Reference to symbol in external test package?
+	// Short-circuit: see comment in wholePackage.
+	if strings.HasSuffix(string(pkgpath), "_test") {
+		return wholePackage(pkg.Types())
+	}
+
+	// package-level symbol?
+	if typesinternal.IsPackageLevel(sym) {
+		return pkgpath, sym.Name(), makeTitle(objectKind(sym), sym.Pkg(), sym.Name())
+	}
+
+	// Inv: sym is field or method, or local.
+	switch sym := sym.(type) {
+	case *types.Func: // => method
+		sig := sym.Signature()
+		isPtr, named := typesinternal.ReceiverNamed(sig.Recv())
+		if named != nil {
+			if !named.Obj().Exported() {
+				return wholePackage(sym.Pkg()) // exported method of unexported type
+			}
+			name := fmt.Sprintf("(%s%s).%s",
+				strings.Repeat("*", btoi(isPtr)), // for *T
+				named.Obj().Name(),
+				sym.Name())
+			fragment := named.Obj().Name() + "." + sym.Name()
+			return pkgpath, fragment, makeTitle("method", sym.Pkg(), name)
+		}
+
+	case *types.Var:
+		if sym.IsField() {
+			// TODO(adonovan): support fields.
+			// The Var symbol doesn't include the struct
+			// type, so we need to use the logic from
+			// Hover. (This isn't important for
+			// DocFragment as fields don't have fragments,
+			// but it matters to the grand unification of
+			// Hover/Definition/DocFragment.
+		}
+	}
+
+	// Field, non-exported method, or local declaration:
+	// just show current package.
+	return wholePackage(pkg.Types())
+}
+
+// thing describes the package or symbol denoted by a selection.
+//
+// TODO(adonovan): Hover, Definition, and References all start by
+// identifying the selected object. Let's achieve a better factoring
+// of the common parts using this structure, including uniform
+// treatment of doc links, linkname, and suchlike.
+type thing struct {
+	// At most one of these fields is set.
+	// (The 'enclosing' field is a fallback for when neither
+	// of the first two is set.)
+	symbol    types.Object   // referenced symbol
+	pkg       *types.Package // referenced package
+	enclosing types.Object   // package-level symbol or method decl enclosing selection
+}
+
+func thingAtPoint(pkg *cache.Package, pgf *parsego.File, start, end token.Pos) thing {
+	path, _ := astutil.PathEnclosingInterval(pgf.File, start, end)
+
+	// In an import spec?
+	if len(path) >= 3 { // [...ImportSpec GenDecl File]
+		if spec, ok := path[len(path)-3].(*ast.ImportSpec); ok {
+			if pkgname := pkg.TypesInfo().PkgNameOf(spec); pkgname != nil {
+				return thing{pkg: pkgname.Imported()}
+			}
+		}
+	}
+
+	// Definition or reference to symbol?
+	var obj types.Object
+	if id, ok := path[0].(*ast.Ident); ok {
+		obj = pkg.TypesInfo().ObjectOf(id)
+
+		// Treat use to PkgName like ImportSpec.
+		if pkgname, ok := obj.(*types.PkgName); ok {
+			return thing{pkg: pkgname.Imported()}
+		}
+
+	} else if sel, ok := path[0].(*ast.SelectorExpr); ok {
+		// e.g. selection is "fmt.Println" or just a portion ("mt.Prin")
+		obj = pkg.TypesInfo().Uses[sel.Sel]
+	}
+	if obj != nil {
+		return thing{symbol: obj}
+	}
+
+	// Find enclosing declaration.
+	if n := len(path); n > 1 {
+		switch decl := path[n-2].(type) {
+		case *ast.FuncDecl:
+			// method?
+			if fn := pkg.TypesInfo().Defs[decl.Name]; fn != nil {
+				return thing{enclosing: fn}
+			}
+
+		case *ast.GenDecl:
+			// path=[... Spec? GenDecl File]
+			for _, spec := range decl.Specs {
+				if n > 2 && spec == path[n-3] {
+					var name *ast.Ident
+					switch spec := spec.(type) {
+					case *ast.ValueSpec:
+						// var, const: use first name
+						name = spec.Names[0]
+					case *ast.TypeSpec:
+						name = spec.Name
+					}
+					if name != nil {
+						return thing{enclosing: pkg.TypesInfo().Defs[name]}
+					}
+					break
+				}
+			}
+		}
+	}
+
+	return thing{} // nothing to see here
+}
+
+// Web is an abstraction of gopls' web server.
+type Web interface {
+	// PkgURL forms URLs of package or symbol documentation.
+	PkgURL(viewID string, path PackagePath, fragment string) protocol.URI
+
+	// SrcURL forms URLs that cause the editor to open a file at a specific position.
+	SrcURL(filename string, line, col8 int) protocol.URI
+}
+
+// PackageDocHTML formats the package documentation page.
 //
 // The posURL function returns a URL that when visited, has the side
 // effect of causing gopls to direct the client editor to navigate to
 // the specified file/line/column position, in UTF-8 coordinates.
 //
-// The pkgURL function returns a URL for the documentation of the
-// specified package and symbol.
-func RenderPackageDoc(pkg *cache.Package, posURL func(filename string, line, col8 int) protocol.URI, pkgURL func(path PackagePath, fragment string) protocol.URI) ([]byte, error) {
+// TODO(adonovan): this function could use some unit tests; we
+// shouldn't have to use integration tests to cover microdetails of
+// HTML rendering. (It is tempting to abstract this function so that
+// it depends only on FileSet/File/Types/TypeInfo/etc, but we should
+// bend the tests to the production interfaces, not the other way
+// around.)
+func PackageDocHTML(viewID string, pkg *cache.Package, web Web) ([]byte, error) {
 	// We can't use doc.NewFromFiles (even with doc.PreserveAST
 	// mode) as it calls ast.NewPackage which assumes that each
 	// ast.File has an ast.Scope and resolves identifiers to
@@ -73,7 +288,7 @@ func RenderPackageDoc(pkg *cache.Package, posURL func(filename string, line, col
 	// TODO(adonovan): simulate that too.
 	fileMap := make(map[string]*ast.File)
 	for _, f := range pkg.Syntax() {
-		fileMap[pkg.FileSet().File(f.Pos()).Name()] = f
+		fileMap[pkg.FileSet().File(f.FileStart).Name()] = f
 	}
 	astpkg := &ast.Package{
 		Name:  pkg.Types().Name(),
@@ -117,71 +332,34 @@ func RenderPackageDoc(pkg *cache.Package, posURL func(filename string, line, col
 		})
 	}
 
-	var docHTML func(comment string) []byte
+	// docHTML renders the doc comment as Markdown.
+	// The fileNode is used to deduce the enclosing file
+	// for the correct import mapping.
+	//
+	// It is not concurrency-safe.
+	var docHTML func(fileNode ast.Node, comment string) []byte
 	{
 		// Adapt doc comment parser and printer
 		// to our representation of Go packages
 		// so that doc links (e.g. "[fmt.Println]")
 		// become valid links.
-
-		printer := docpkg.Printer()
-		printer.DocLinkURL = func(link *comment.DocLink) string {
-			path := pkg.Metadata().PkgPath
-			if link.ImportPath != "" {
-				path = PackagePath(link.ImportPath)
-			}
-			fragment := link.Name
-			if link.Recv != "" {
-				fragment = link.Recv + "." + link.Name
-			}
-			return pkgURL(path, fragment)
-		}
-		parser := docpkg.Parser()
-		parser.LookupPackage = func(name string) (importPath string, ok bool) {
-			// Ambiguous: different files in the same
-			// package may have different import mappings,
-			// but the hook doesn't provide the file context.
-			// TODO(adonovan): conspire with docHTML to
-			// pass the doc comment's enclosing file through
-			// a shared variable, so that we can compute
-			// the correct per-file mapping.
-			//
-			// TODO(adonovan): check for PkgName.Name
-			// matches, but also check for
-			// PkgName.Imported.Namer matches, since some
-			// packages are typically imported under a
-			// non-default name (e.g. pathpkg "path") but
-			// may be referred to in doc links using their
-			// canonical name.
-			for _, f := range pkg.Syntax() {
-				for _, imp := range f.Imports {
-					pkgName, ok := typesutil.ImportedPkgName(pkg.TypesInfo(), imp)
-					if ok && pkgName.Name() == name {
-						return pkgName.Imported().Path(), true
-					}
+		printer := &comment.Printer{
+			DocLinkURL: func(link *comment.DocLink) string {
+				path := pkg.Metadata().PkgPath
+				if link.ImportPath != "" {
+					path = PackagePath(link.ImportPath)
 				}
-			}
-			return "", false
+				fragment := link.Name
+				if link.Recv != "" {
+					fragment = link.Recv + "." + link.Name
+				}
+				return web.PkgURL(viewID, path, fragment)
+			},
 		}
-		parser.LookupSym = func(recv, name string) (ok bool) {
-			defer func() {
-				log.Printf("LookupSym %q %q = %t ", recv, name, ok)
-			}()
-			// package-level decl?
-			if recv == "" {
-				return pkg.Types().Scope().Lookup(name) != nil
-			}
-
-			// method?
-			tname, ok := pkg.Types().Scope().Lookup(recv).(*types.TypeName)
-			if !ok {
-				return false
-			}
-			m, _, _ := types.LookupFieldOrMethod(tname.Type(), true, pkg.Types(), name)
-			return is[*types.Func](m)
-		}
-		docHTML = func(comment string) []byte {
-			return printer.HTML(parser.Parse(comment))
+		parse := newDocCommentParser(pkg)
+		docHTML = func(fileNode ast.Node, comment string) []byte {
+			doc := parse(fileNode, comment)
+			return printer.HTML(doc)
 		}
 	}
 
@@ -196,39 +374,46 @@ func RenderPackageDoc(pkg *cache.Package, posURL func(filename string, line, col
 <html>
 <head>
   <meta charset="UTF-8">
-  <style>` + pkgDocStyle + `</style>
   <title>` + title + `</title>
-  <script type='text/javascript'>
-// httpGET requests a URL for its effects only.
-function httpGET(url) {
-	var xhttp = new XMLHttpRequest();
-	xhttp.open("GET", url, true);
-	xhttp.send();
-	return false; // disable usual <a href=...> behavior
+  <link rel="stylesheet" href="/assets/common.css">
+  <script src="/assets/common.js"></script>
+  <style>
+.lit { color: darkgreen; }
+
+header {
+  position: sticky;
+  top: 0;
+  left: 0;
+  width: 100%;
+  padding: 0.3em;
 }
 
-window.onload = () => {
+.Documentation-sinceVersion {
+  font-weight: normal;
+  color: #808080;
+  float: right;
+}
+
+#pkgsite { height: 1.5em; }
+
+#hdr-Selector {
+  margin-right: 0.3em;
+  float: right;
+  min-width: 25em;
+  padding: 0.3em;
+}
+  </style>
+  <script type='text/javascript'>
+window.addEventListener('load', function() {
 	// Hook up the navigation selector.
 	document.getElementById('hdr-Selector').onchange = (e) => {
 		window.location.href = e.target.value;
 	};
-};
-
-// Start a GET /hang request. If it ever completes, the server
-// has disconnected. Show a banner in that case.
-{
-	var x = new XMLHttpRequest();
-	x.open("GET", "/hang", true);
-	x.onloadend = () => {
-		document.getElementById("disconnected").style.display = 'block';
-	};
-	x.send();
-};
+});
   </script>
 </head>
 <body>
 <header>
-<div id='disconnected'>Gopls server has terminated. Page is inactive.</div>
 <select id='hdr-Selector'>
 <optgroup label="Documentation">
   <option label="Overview" value="#hdr-Overview"/>
@@ -256,7 +441,7 @@ window.onload = () => {
 		label := obj.Name() // for a type
 		if fn, ok := obj.(*types.Func); ok {
 			var buf strings.Builder
-			sig := fn.Type().(*types.Signature)
+			sig := fn.Signature()
 			if sig.Recv() != nil {
 				fmt.Fprintf(&buf, "(%s) ", sig.Recv().Name())
 				fragment = recvType + "." + fn.Name()
@@ -313,26 +498,6 @@ window.onload = () => {
 
 	// -- main element --
 
-	// sourceLink returns HTML for a link to open a file in the client editor.
-	sourceLink := func(text, url string) string {
-		// The /open URL returns nothing but has the side effect
-		// of causing the LSP client to open the requested file.
-		// So we use onclick to prevent the browser from navigating.
-		// We keep the href attribute as it causes the <a> to render
-		// as a link: blue, underlined, with URL hover information.
-		return fmt.Sprintf(`<a href="%[1]s" onclick='return httpGET("%[1]s")'>%[2]s</a>`,
-			escape(url), text)
-	}
-
-	// objHTML returns HTML for obj.Name(), possibly as a link.
-	objHTML := func(obj types.Object) string {
-		text := obj.Name()
-		if posn := safetoken.StartPosition(pkg.FileSet(), obj.Pos()); posn.IsValid() {
-			return sourceLink(text, posURL(posn.Filename, posn.Line, posn.Column))
-		}
-		return text
-	}
-
 	// nodeHTML returns HTML markup for a syntax tree.
 	// It replaces referring identifiers with links,
 	// and adds style spans for strings and comments.
@@ -344,7 +509,7 @@ window.onload = () => {
 				// imported package name?
 				if pkgname, ok := obj.(*types.PkgName); ok {
 					// TODO(adonovan): do this for Defs of PkgName too.
-					return pkgURL(PackagePath(pkgname.Imported().Path()), "")
+					return web.PkgURL(viewID, PackagePath(pkgname.Imported().Path()), "")
 				}
 
 				// package-level symbol?
@@ -352,18 +517,18 @@ window.onload = () => {
 					if obj.Pkg() == pkg.Types() {
 						return "#" + obj.Name() // intra-package ref
 					} else {
-						return pkgURL(PackagePath(obj.Pkg().Path()), obj.Name())
+						return web.PkgURL(viewID, PackagePath(obj.Pkg().Path()), obj.Name())
 					}
 				}
 
 				// method of package-level named type?
 				if fn, ok := obj.(*types.Func); ok {
-					sig := fn.Type().(*types.Signature)
+					sig := fn.Signature()
 					if sig.Recv() != nil {
 						_, named := typesinternal.ReceiverNamed(sig.Recv())
 						if named != nil {
 							fragment := named.Obj().Name() + "." + fn.Name()
-							return pkgURL(PackagePath(fn.Pkg().Path()), fragment)
+							return web.PkgURL(viewID, PackagePath(fn.Pkg().Path()), fragment)
 						}
 					}
 					return ""
@@ -392,7 +557,7 @@ window.onload = () => {
 		// appear as separate decls. We should too.
 		var buf bytes.Buffer
 		for _, file := range pkg.CompiledGoFiles() {
-			if astutil.NodeContains(file.File, n.Pos()) {
+			if goplsastutil.NodeContains(file.File, n.Pos()) {
 				pos := n.Pos()
 
 				// emit emits source in the interval [pos:to] and updates pos.
@@ -453,15 +618,9 @@ window.onload = () => {
 	//   since it is unreachable for the methods in go/doc.
 	// - elides parameters after the first three: f(a, b, c, ...).
 	fnString := func(fn *types.Func) string {
-		// pkgRelative qualifies types by package name alone
-		pkgRelative := func(other *types.Package) string {
-			if pkg.Types() == other {
-				return "" // same package; unqualified
-			}
-			return other.Name()
-		}
+		pkgRelative := typesinternal.NameRelativeTo(pkg.Types())
 
-		sig := fn.Type().(*types.Signature)
+		sig := fn.Signature()
 
 		// Emit "func (recv T) F".
 		var buf bytes.Buffer
@@ -490,15 +649,26 @@ window.onload = () => {
 		// parameters 4+ with "invalid type", format,
 		// then post-process the string.
 		if sig.Params().Len() > 3 {
+
+			// Clone each TypeParam as NewSignatureType modifies them (#67294).
+			cloneTparams := func(seq *types.TypeParamList) []*types.TypeParam {
+				slice := make([]*types.TypeParam, seq.Len())
+				for i := range slice {
+					tparam := seq.At(i)
+					slice[i] = types.NewTypeParam(tparam.Obj(), tparam.Constraint())
+				}
+				return slice
+			}
+
 			sig = types.NewSignatureType(
 				sig.Recv(),
-				typesSeqToSlice[*types.TypeParam](sig.RecvTypeParams()),
-				typesSeqToSlice[*types.TypeParam](sig.TypeParams()),
+				cloneTparams(sig.RecvTypeParams()),
+				cloneTparams(sig.TypeParams()),
 				types.NewTuple(append(
-					typesSeqToSlice[*types.Var](sig.Params())[:3],
-					types.NewVar(0, nil, "", types.Typ[types.Invalid]))...),
+					slices.Collect(sig.Params().Variables())[:3],
+					types.NewParam(0, nil, "", types.Typ[types.Invalid]))...),
 				sig.Results(),
-				sig.Variadic())
+				false) // any final ...T parameter is truncated
 		}
 		types.WriteSignature(&buf, sig, pkgRelative)
 		return strings.ReplaceAll(buf.String(), ", invalid type)", ", ...)")
@@ -517,7 +687,12 @@ window.onload = () => {
 		"https://pkg.go.dev/"+string(pkg.Types().Path()))
 
 	// package doc
-	fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n", docHTML(docpkg.Doc))
+	for _, f := range pkg.Syntax() {
+		if f.Doc != nil {
+			fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n", docHTML(f.Doc, docpkg.Doc))
+			break
+		}
+	}
 
 	// symbol index
 	fmt.Fprintf(&buf, "<h2 id='hdr-Index'>Index</h2>\n")
@@ -575,7 +750,7 @@ window.onload = () => {
 			fmt.Fprintf(&buf, "<pre class='code'>%s</pre>\n", nodeHTML(&decl2))
 
 			// comment (if any)
-			fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n", docHTML(v.Doc))
+			fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n", docHTML(v.Decl, v.Doc))
 		}
 	}
 	fmt.Fprintf(&buf, "<h2 id='hdr-Constants'>Constants</h2>\n")
@@ -591,6 +766,15 @@ window.onload = () => {
 		values(docpkg.Vars)
 	}
 
+	// addedInHTML returns an HTML division containing the Go release version at
+	// which this obj became available.
+	addedInHTML := func(obj types.Object) string {
+		if sym := StdSymbolOf(obj); sym != nil && sym.Version != stdlib.Version(0) {
+			return fmt.Sprintf("<span class='Documentation-sinceVersion'>added in %v</span>", sym.Version)
+		}
+		return ""
+	}
+
 	// package-level functions
 	fmt.Fprintf(&buf, "<h2 id='hdr-Functions'>Functions</h2>\n")
 	// funcs emits a list of package-level functions,
@@ -598,15 +782,16 @@ window.onload = () => {
 	funcs := func(funcs []*doc.Func) {
 		for _, docfn := range funcs {
 			obj := scope.Lookup(docfn.Name).(*types.Func)
-			fmt.Fprintf(&buf, "<h3 id='%s'>func %s</h3>\n",
-				docfn.Name, objHTML(obj))
+
+			fmt.Fprintf(&buf, "<h3 id='%s'>func %s %s</h3>\n",
+				docfn.Name, objHTML(pkg.FileSet(), web, obj), addedInHTML(obj))
 
 			// decl: func F(params) results
 			fmt.Fprintf(&buf, "<pre class='code'>%s</pre>\n",
 				nodeHTML(docfn.Decl.Type))
 
 			// comment (if any)
-			fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n", docHTML(docfn.Doc))
+			fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n", docHTML(docfn.Decl, docfn.Doc))
 		}
 	}
 	funcs(docpkg.Funcs)
@@ -617,7 +802,8 @@ window.onload = () => {
 		tname := scope.Lookup(doctype.Name).(*types.TypeName)
 
 		// title and source link
-		fmt.Fprintf(&buf, "<h3 id='%s'>type %s</a></h3>\n", doctype.Name, objHTML(tname))
+		fmt.Fprintf(&buf, "<h3 id='%s'>type %s %s</h3>\n",
+			doctype.Name, objHTML(pkg.FileSet(), web, tname), addedInHTML(tname))
 
 		// declaration
 		// TODO(adonovan): excise non-exported struct fields somehow.
@@ -626,7 +812,7 @@ window.onload = () => {
 		fmt.Fprintf(&buf, "<pre class='code'>%s</pre>\n", nodeHTML(&decl2))
 
 		// comment (if any)
-		fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n", docHTML(doctype.Doc))
+		fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n", docHTML(doctype.Decl, doctype.Doc))
 
 		// subelements
 		values(doctype.Consts) // constants of type T
@@ -636,9 +822,10 @@ window.onload = () => {
 		// methods on T
 		for _, docmethod := range doctype.Methods {
 			method, _, _ := types.LookupFieldOrMethod(tname.Type(), true, tname.Pkg(), docmethod.Name)
-			fmt.Fprintf(&buf, "<h4 id='%s.%s'>func (%s) %s</h4>\n",
+			fmt.Fprintf(&buf, "<h4 id='%s.%s'>func (%s) %s %s</h4>\n",
 				doctype.Name, docmethod.Name,
-				doctype.Name, objHTML(method))
+				docmethod.Orig, // T or *T
+				objHTML(pkg.FileSet(), web, method), addedInHTML(method))
 
 			// decl: func (x T) M(params) results
 			fmt.Fprintf(&buf, "<pre class='code'>%s</pre>\n",
@@ -646,7 +833,7 @@ window.onload = () => {
 
 			// comment (if any)
 			fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n",
-				docHTML(docmethod.Doc))
+				docHTML(docmethod.Decl, docmethod.Doc))
 		}
 	}
 
@@ -654,7 +841,7 @@ window.onload = () => {
 	fmt.Fprintf(&buf, "<h2 id='hdr-SourceFiles'>Source files</h2>\n")
 	for _, filename := range docpkg.Filenames {
 		fmt.Fprintf(&buf, "<div class='comment'>%s</div>\n",
-			sourceLink(filepath.Base(filename), posURL(filename, 1, 1)))
+			sourceLink(filepath.Base(filename), web.SrcURL(filename, 1, 1)))
 	}
 
 	fmt.Fprintf(&buf, "</main>\n")
@@ -663,151 +850,3 @@ window.onload = () => {
 
 	return buf.Bytes(), nil
 }
-
-// typesSeq abstracts various go/types sequence types:
-// MethodSet, Tuple, TypeParamList, TypeList.
-// TODO(adonovan): replace with go1.23 iterators.
-type typesSeq[T any] interface {
-	Len() int
-	At(int) T
-}
-
-func typesSeqToSlice[T any](seq typesSeq[T]) []T {
-	slice := make([]T, seq.Len())
-	for i := range slice {
-		slice[i] = seq.At(i)
-	}
-	return slice
-}
-
-// (partly taken from pkgsite's typography.css)
-const pkgDocStyle = `
-body {
-  font-family: Helvetica, Arial, sans-serif;
-  font-size: 1rem;
-  line-height: normal;
-}
-
-h1 {
-  font-size: 1.5rem;
-}
-
-h2 {
-  font-size: 1.375rem;
-}
-
-h3 {
-  font-size: 1.25rem;
-}
-
-h4 {
-  font-size: 1.125rem;
-}
-
-h5 {
-  font-size: 1rem;
-}
-
-h6 {
-  font-size: 0.875rem;
-}
-
-h1,
-h2,
-h3,
-h4 {
-  font-weight: 600;
-  line-height: 1.25em;
-  word-break: break-word;
-}
-
-h5,
-h6 {
-  font-weight: 500;
-  line-height: 1.3em;
-  word-break: break-word;
-}
-
-p {
-  font-size: 1rem;
-  line-height: 1.5rem;
-  max-width: 60rem;
-}
-
-strong {
-  font-weight: 600;
-}
-
-code,
-pre,
-textarea.code {
-  font-family: Consolas, 'Liberation Mono', Menlo, monospace;
-  font-size: 0.875rem;
-  line-height: 1.5em;
-}
-
-pre,
-textarea.code {
-  background-color: #eee;
-  border: 3px;
-  border-radius: 3px
-  color: black;
-  overflow-x: auto;
-  padding: 0.625rem;
-  tab-size: 4;
-  white-space: pre;
-}
-
-button,
-input,
-select,
-textarea {
-  font: inherit;
-}
-
-a,
-a:link,
-a:visited {
-  color: rgb(0, 125, 156);
-  text-decoration: none;
-}
-
-a:hover,
-a:focus {
-  color: rgb(0, 125, 156);
-  text-decoration: underline;
-}
-
-a:hover > * {
-  text-decoration: underline;
-}
-
-.lit { color: darkgreen; }
-
-#pkgsite { height: 1.5em; }
-
-header {
-  position: sticky;
-  top: 0;
-  left: 0;
-  width: 100%;
-  padding: 0.3em;
-}
-
-#hdr-Selector {
-  margin-right: 0.3em;
-  float: right;
-  min-width: 25em;
-  padding: 0.3em;
-}
-
-#disconnected {
-  position: fixed;
-  top: 1em;
-  left: 1em;
-  display: none; /* initially */
-  background-color: white;
-  border: thick solid red;
-  padding: 2em;
-}
-`
