@@ -49,6 +49,7 @@ import (
 	"golang.org/x/tools/gopls/internal/analysis/deprecated"
 	"golang.org/x/tools/gopls/internal/analysis/embeddirective"
 	"golang.org/x/tools/gopls/internal/analysis/fillreturns"
+	"golang.org/x/tools/gopls/internal/analysis/hostport"
 	"golang.org/x/tools/gopls/internal/analysis/infertypeargs"
 	"golang.org/x/tools/gopls/internal/analysis/modernize"
 	"golang.org/x/tools/gopls/internal/analysis/nonewvars"
@@ -61,6 +62,7 @@ import (
 	"golang.org/x/tools/gopls/internal/analysis/unusedvariable"
 	"golang.org/x/tools/gopls/internal/analysis/yield"
 	"golang.org/x/tools/gopls/internal/protocol"
+	"golang.org/x/tools/internal/gofix"
 )
 
 // Analyzer augments a [analysis.Analyzer] with additional LSP configuration.
@@ -68,7 +70,7 @@ import (
 // Analyzers are immutable, since they are shared across multiple LSP sessions.
 type Analyzer struct {
 	analyzer    *analysis.Analyzer
-	enabled     bool
+	nonDefault  bool
 	actionKinds []protocol.CodeActionKind
 	severity    protocol.DiagnosticSeverity
 	tags        []protocol.DiagnosticTag
@@ -79,7 +81,7 @@ func (a *Analyzer) Analyzer() *analysis.Analyzer { return a.analyzer }
 
 // EnabledByDefault reports whether the analyzer is enabled by default for all sessions.
 // This value can be configured per-analysis in user settings.
-func (a *Analyzer) EnabledByDefault() bool { return a.enabled }
+func (a *Analyzer) EnabledByDefault() bool { return !a.nonDefault }
 
 // ActionKinds is the set of kinds of code action this analyzer produces.
 //
@@ -87,12 +89,35 @@ func (a *Analyzer) EnabledByDefault() bool { return a.enabled }
 // TODO(rfindley): revisit.
 func (a *Analyzer) ActionKinds() []protocol.CodeActionKind { return a.actionKinds }
 
-// Severity is the severity set for diagnostics reported by this
-// analyzer. If left unset it defaults to Warning.
+// Severity is the severity set for diagnostics reported by this analyzer.
+// The default severity is SeverityWarning.
 //
-// Note: diagnostics with severity protocol.SeverityHint do not show up in
-// the VS Code "problems" tab.
-func (a *Analyzer) Severity() protocol.DiagnosticSeverity { return a.severity }
+// While the LSP spec does not specify how severity should be used, here are
+// some guiding heuristics:
+//   - Error: for parse and type errors, which would stop the build.
+//   - Warning: for analyzer diagnostics reporting likely bugs.
+//   - Info: for analyzer diagnostics that do not indicate bugs, but may
+//     suggest inaccurate or superfluous code.
+//   - Hint: for analyzer diagnostics that do not indicate mistakes, but offer
+//     simplifications or modernizations. By their nature, hints should
+//     generally carry quick fixes.
+//
+// The difference between Info and Hint is particularly subtle. Importantly,
+// Hint diagnostics do not appear in the Problems tab in VS Code, so they are
+// less intrusive than Info diagnostics. The rule of thumb is this: use Info if
+// the diagnostic is not a bug, but the author probably didn't mean to write
+// the code that way. Use Hint if the diagnostic is not a bug and the author
+// indended to write the code that way, but there is a simpler or more modern
+// way to express the same logic. An 'unused' diagnostic is Info level, since
+// the author probably didn't mean to check in unreachable code. A 'modernize'
+// or 'deprecated' diagnostic is Hint level, since the author intended to write
+// the code that way, but now there is a better way.
+func (a *Analyzer) Severity() protocol.DiagnosticSeverity {
+	if a.severity == 0 {
+		return protocol.SeverityWarning
+	}
+	return a.severity
+}
 
 // Tags is extra tags (unnecessary, deprecated, etc) for diagnostics
 // reported by this analyzer.
@@ -108,85 +133,99 @@ func (a *Analyzer) String() string { return a.analyzer.String() }
 var DefaultAnalyzers = make(map[string]*Analyzer) // initialized below
 
 func init() {
-	// The traditional vet suite:
+	// See [Analyzer.Severity] for guidance on setting analyzer severity below.
 	analyzers := []*Analyzer{
 		// The traditional vet suite:
-		{analyzer: appends.Analyzer, enabled: true},
-		{analyzer: asmdecl.Analyzer, enabled: true},
-		{analyzer: assign.Analyzer, enabled: true},
-		{analyzer: atomic.Analyzer, enabled: true},
-		{analyzer: bools.Analyzer, enabled: true},
-		{analyzer: buildtag.Analyzer, enabled: true},
-		{analyzer: cgocall.Analyzer, enabled: true},
-		{analyzer: composite.Analyzer, enabled: true},
-		{analyzer: copylock.Analyzer, enabled: true},
-		{analyzer: defers.Analyzer, enabled: true},
-		{analyzer: deprecated.Analyzer, enabled: true, severity: protocol.SeverityHint, tags: []protocol.DiagnosticTag{protocol.Deprecated}},
-		{analyzer: directive.Analyzer, enabled: true},
-		{analyzer: errorsas.Analyzer, enabled: true},
-		{analyzer: framepointer.Analyzer, enabled: true},
-		{analyzer: httpresponse.Analyzer, enabled: true},
-		{analyzer: ifaceassert.Analyzer, enabled: true},
-		{analyzer: loopclosure.Analyzer, enabled: true},
-		{analyzer: lostcancel.Analyzer, enabled: true},
-		{analyzer: nilfunc.Analyzer, enabled: true},
-		{analyzer: printf.Analyzer, enabled: true},
-		{analyzer: shift.Analyzer, enabled: true},
-		{analyzer: sigchanyzer.Analyzer, enabled: true},
-		{analyzer: slog.Analyzer, enabled: true},
-		{analyzer: stdmethods.Analyzer, enabled: true},
-		{analyzer: stdversion.Analyzer, enabled: true},
-		{analyzer: stringintconv.Analyzer, enabled: true},
-		{analyzer: structtag.Analyzer, enabled: true},
-		{analyzer: testinggoroutine.Analyzer, enabled: true},
-		{analyzer: tests.Analyzer, enabled: true},
-		{analyzer: timeformat.Analyzer, enabled: true},
-		{analyzer: unmarshal.Analyzer, enabled: true},
-		{analyzer: unreachable.Analyzer, enabled: true},
-		{analyzer: unsafeptr.Analyzer, enabled: true},
-		{analyzer: unusedresult.Analyzer, enabled: true},
+		{analyzer: appends.Analyzer},
+		{analyzer: asmdecl.Analyzer},
+		{analyzer: assign.Analyzer},
+		{analyzer: atomic.Analyzer},
+		{analyzer: bools.Analyzer},
+		{analyzer: buildtag.Analyzer},
+		{analyzer: cgocall.Analyzer},
+		{analyzer: composite.Analyzer},
+		{analyzer: copylock.Analyzer},
+		{analyzer: defers.Analyzer},
+		{analyzer: deprecated.Analyzer, severity: protocol.SeverityHint, tags: []protocol.DiagnosticTag{protocol.Deprecated}},
+		{analyzer: directive.Analyzer},
+		{analyzer: errorsas.Analyzer},
+		{analyzer: framepointer.Analyzer},
+		{analyzer: httpresponse.Analyzer},
+		{analyzer: ifaceassert.Analyzer},
+		{analyzer: loopclosure.Analyzer},
+		{analyzer: lostcancel.Analyzer},
+		{analyzer: nilfunc.Analyzer},
+		{analyzer: printf.Analyzer},
+		{analyzer: shift.Analyzer},
+		{analyzer: sigchanyzer.Analyzer},
+		{analyzer: slog.Analyzer},
+		{analyzer: stdmethods.Analyzer},
+		{analyzer: stdversion.Analyzer},
+		{analyzer: stringintconv.Analyzer},
+		{analyzer: structtag.Analyzer},
+		{analyzer: testinggoroutine.Analyzer},
+		{analyzer: tests.Analyzer},
+		{analyzer: timeformat.Analyzer},
+		{analyzer: unmarshal.Analyzer},
+		{analyzer: unreachable.Analyzer},
+		{analyzer: unsafeptr.Analyzer},
+		{analyzer: unusedresult.Analyzer},
 
 		// not suitable for vet:
 		// - some (nilness, yield) use go/ssa; see #59714.
 		// - others don't meet the "frequency" criterion;
 		//   see GOROOT/src/cmd/vet/README.
-		// - some (modernize) report diagnostics on perfectly valid code (hence severity=info)
-		{analyzer: atomicalign.Analyzer, enabled: true},
-		{analyzer: deepequalerrors.Analyzer, enabled: true},
-		{analyzer: nilness.Analyzer, enabled: true}, // uses go/ssa
-		{analyzer: yield.Analyzer, enabled: true},   // uses go/ssa
-		{analyzer: sortslice.Analyzer, enabled: true},
-		{analyzer: embeddirective.Analyzer, enabled: true},
-		{analyzer: waitgroup.Analyzer, enabled: true}, // to appear in cmd/vet@go1.25
-		{analyzer: modernize.Analyzer, enabled: true, severity: protocol.SeverityInformation},
+		{analyzer: atomicalign.Analyzer},
+		{analyzer: deepequalerrors.Analyzer},
+		{analyzer: nilness.Analyzer}, // uses go/ssa
+		{analyzer: yield.Analyzer},   // uses go/ssa
+		{analyzer: sortslice.Analyzer},
+		{analyzer: embeddirective.Analyzer},
+		{analyzer: waitgroup.Analyzer}, // to appear in cmd/vet@go1.25
+		{analyzer: hostport.Analyzer},  // to appear in cmd/vet@go1.25
 
 		// disabled due to high false positives
-		{analyzer: shadow.Analyzer, enabled: false}, // very noisy
+		{analyzer: shadow.Analyzer, nonDefault: true}, // very noisy
 		// fieldalignment is not even off-by-default; see #67762.
 
-		// "simplifiers": analyzers that offer mere style fixes
-		// gofmt -s suite:
-		{analyzer: simplifycompositelit.Analyzer, enabled: true, actionKinds: []protocol.CodeActionKind{protocol.SourceFixAll, protocol.QuickFix}},
-		{analyzer: simplifyrange.Analyzer, enabled: true, actionKinds: []protocol.CodeActionKind{protocol.SourceFixAll, protocol.QuickFix}},
-		{analyzer: simplifyslice.Analyzer, enabled: true, actionKinds: []protocol.CodeActionKind{protocol.SourceFixAll, protocol.QuickFix}},
-		// other simplifiers:
-		{analyzer: infertypeargs.Analyzer, enabled: true, severity: protocol.SeverityHint},
-		{analyzer: unusedparams.Analyzer, enabled: true},
-		{analyzer: unusedfunc.Analyzer, enabled: true},
-		{analyzer: unusedwrite.Analyzer, enabled: true}, // uses go/ssa
+		// simplifiers and modernizers
+		//
+		// These analyzers offer mere style fixes on correct code,
+		// thus they will never appear in cmd/vet and
+		// their severity level is "information".
+		//
+		// gofmt -s suite
+		{
+			analyzer:    simplifycompositelit.Analyzer,
+			actionKinds: []protocol.CodeActionKind{protocol.SourceFixAll, protocol.QuickFix},
+			severity:    protocol.SeverityInformation,
+		},
+		{
+			analyzer:    simplifyrange.Analyzer,
+			actionKinds: []protocol.CodeActionKind{protocol.SourceFixAll, protocol.QuickFix},
+			severity:    protocol.SeverityInformation,
+		},
+		{
+			analyzer:    simplifyslice.Analyzer,
+			actionKinds: []protocol.CodeActionKind{protocol.SourceFixAll, protocol.QuickFix},
+			severity:    protocol.SeverityInformation,
+		},
+		// other simplifiers
+		{analyzer: gofix.Analyzer, severity: protocol.SeverityHint},
+		{analyzer: infertypeargs.Analyzer, severity: protocol.SeverityInformation},
+		{analyzer: unusedparams.Analyzer, severity: protocol.SeverityInformation},
+		{analyzer: unusedfunc.Analyzer, severity: protocol.SeverityInformation},
+		{analyzer: unusedwrite.Analyzer, severity: protocol.SeverityInformation}, // uses go/ssa
+		{analyzer: modernize.Analyzer, severity: protocol.SeverityHint},
 
 		// type-error analyzers
 		// These analyzers enrich go/types errors with suggested fixes.
-		{analyzer: fillreturns.Analyzer, enabled: true},
-		{analyzer: nonewvars.Analyzer, enabled: true},
-		{analyzer: noresultvalues.Analyzer, enabled: true},
-		// TODO(rfindley): why isn't the 'unusedvariable' analyzer enabled, if it
-		// is only enhancing type errors with suggested fixes?
-		//
-		// In particular, enabling this analyzer could cause unused variables to be
-		// greyed out, (due to the 'deletions only' fix). That seems like a nice UI
-		// feature.
-		{analyzer: unusedvariable.Analyzer, enabled: false},
+		// Since they exist only to attach their fixes to type errors, their
+		// severity is irrelevant.
+		{analyzer: fillreturns.Analyzer},
+		{analyzer: nonewvars.Analyzer},
+		{analyzer: noresultvalues.Analyzer},
+		{analyzer: unusedvariable.Analyzer},
 	}
 	for _, analyzer := range analyzers {
 		DefaultAnalyzers[analyzer.analyzer.Name] = analyzer
